#
# Copyright 2009, Red Hat, Inc.
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
#
# Author: Will Woods <wwoods@redhat.com>

from autotest_lib.client.bin import test, utils
from autotest_lib.client.bin.test_config import config_loader
from autotest_lib.client.common_lib import error
import autoqa.koji_utils, autoqa.util
import os

class rpmlint(test.test):
    version = 1 # increment if setup() changes

    def initialize(self, nvr, config):
        self.config = config_loader(config, self.tmpdir)

    def setup(self):
        utils.system('yum -y install rpmlint')

    def run_once(self, nvr):
        koji = autoqa.koji_utils.SimpleKojiClientSession()
        pkgurls = koji.nvr_to_urls(nvr)
        rpmdir = os.path.join(self.tmpdir, 'rpms')
        os.makedirs(rpmdir)
        print "Saving RPMs to %s" % rpmdir
        for p in pkgurls:
            # fetch package to rpmdir
            print "Grabbing %s" % p
            localfile = os.path.join(rpmdir, os.path.basename(p))
            autoqa.util.grabber.urlgrab(p, localfile)
        cmd = 'rpmlint %s' % rpmdir 
        result = 'PASS'
        try:
            self.results = utils.system_output(cmd+" 2>&1", retain_output=True)
        except error.CmdError, e:
            result = 'FAIL'
            self.results = e.result_obj.stdout

        # TODO filter known/waived errors/warnings
        # - Need to store the filters somewhere authoritative, get signoff, etc
        # TODO email results to package owner/builder
        
        # Write a copy of the output into the resultsdir
        out = open(os.path.join(self.resultsdir,'rpmlint.log'),'wb')
        out.write(self.results)
        out.close()
        
        # email results to mailing list
        result_email = self.config.get('test','result_email')
        if result_email:
            # parse last line for summary ('N errors, N warnings')
            summary = ((self.results.splitlines()[-1]).split(';')[-1])[1:-1]
            print 'Sending email with results to %s ...' % result_email
            utils.send_email(mail_to=result_email,
                mail_from=self.config.get('test','mail_from'),
                mail_server=self.config.get('test','smtpserver'),
                subject='rpmlint: %s for package %s' % \
                    (summary, nvr),
                body = self.results)
        
        # if some command failed, make this test fail
        if result == 'FAIL':
            raise error.TestFail
